"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const provider_1 = require("./provider");
const utils_1 = require("./utils");
const logger_1 = require("./utils/logger");
const logger = logger_1.createLogger('azure.functions.deploy.funcitonDeploy');
class FunctionDeploy {
    constructor(options) {
        this.bundlePath = options.bundlePath;
        this.azure = new provider_1.AzureProvider(options);
    }
    deploy() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.login();
            yield this.cleanUpFunctions();
            yield this.createResourceGroup();
            yield this.createFunctionApp();
            const zipFile = yield this.createZipFile();
            yield this.zipDeploy(zipFile);
        });
    }
    login() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.azure.login();
        });
    }
    cleanUpFunctions() {
        return __awaiter(this, void 0, void 0, function* () {
            const isExisting = yield this.azure.isExistingFunctionApp();
            if (isExisting) {
                logger.debug(`Existing Function App found, trying to clean up...`);
                const functions = yield utils_1.FileHelper.loadFunctionsFromDirectory(this.bundlePath);
                if (functions && functions.size > 0) {
                    yield this.azure.getDeployedFunctionsNames();
                    yield this.azure.cleanUpFunctionsBeforeDeploy(Array.from(functions.keys()));
                }
            }
        });
    }
    createResourceGroup() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.azure.createResourceGroup();
        });
    }
    createFunctionApp() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.azure.createFunctionApp();
        });
    }
    createZipFile() {
        return __awaiter(this, void 0, void 0, function* () {
            yield utils_1.FileHelper.mkDirP('./.bundle');
            return utils_1.FileHelper.zipFolder(this.bundlePath, './.bundle/app.zip');
        });
    }
    zipDeploy(zipFile) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.azure.uploadFunction(zipFile);
        });
    }
}
exports.FunctionDeploy = FunctionDeploy;
//# sourceMappingURL=functionPublish.js.map