#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const path = require("path");
const _1 = require("./");
const utils_1 = require("./utils");
const CONSTANTS_1 = require("./CONSTANTS");
const logger = utils_1.createLogger('azure.functions.webpack.main');
function runCli() {
    return __awaiter(this, void 0, void 0, function* () {
        const p = program
            .version("0.0.1");
        p.command("pack [path]")
            .description("Will pack the specified path or the current directory if none is specified")
            .option("-u, --uglify", "Uglify the project when webpacking")
            .option("-o, --output <path>", "Path for output directory")
            .action(pack);
        p.command("publish <unique-app-name>")
            .option("-d --dir <path>", "bundle directory")
            .option("-l --location <location>", "host loction")
            .description("Will pack the specified path or the current directory if none is specified")
            .action(publish);
        p.command("deploy <name> [path]")
            .option("-o, --output <path>", "Path for output directory")
            .option("-l --location <location>", "host loction")
            .description("Will pack the specified path or the current directory, and publish it to azure with given Function app name")
            .action(deploy);
        p.command("*", null, { noHelp: true, isDefault: true })
            .action(() => {
            p.help();
        });
        p.parse(process.argv);
        if (!process.argv.slice(2).length) {
            p.help();
        }
    });
}
function pack(name, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let config = yield utils_1.ConfigLoader.loadConfig();
        config = config || {
            ignoredModules: [],
        };
        let projectRootPath = "";
        try {
            projectRootPath = name ?
                path.join(process.cwd(), name) : process.cwd();
        }
        catch (error) {
            logger.error(error);
            throw new Error("Could not determine route");
        }
        let uglify = false;
        try {
            if (options.uglify) {
                uglify = true;
            }
        }
        catch (e) {
            logger.error(e);
            throw new Error("Could not parse the uglify option");
        }
        let outputPath = CONSTANTS_1.DEFAULT_OUTPUT;
        try {
            if (options.output) {
                outputPath = options.output;
            }
        }
        catch (e) {
            logger.error(e);
            throw new Error("Could not parse the output option");
        }
        const generator = new _1.PackhostGenerator({
            projectRootPath,
            outputPath,
        });
        try {
            logger.info("Generating project files/metadata");
            yield generator.updateProject();
        }
        catch (error) {
            logger.error(error);
            throw new Error("Could not generate project");
        }
        try {
            logger.info("Webpacking project");
            yield _1.WebpackRunner.run({
                projectRootPath,
                uglify,
                outputPath,
                ignoredModules: config.ignoredModules,
            });
        }
        catch (error) {
            logger.error(error);
            throw new Error("Could not webpack project");
        }
        logger.info("Pack Completed!");
    });
}
function publish(name, options) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            logger.info("Publishing project");
            const bundlePath = options.dir || CONSTANTS_1.DEFAULT_OUTPUT;
            if (!(yield utils_1.FileHelper.exists(bundlePath))) {
                logger.info("The bundle folder is not existing");
                process.exit(1);
            }
            const deploy = new _1.FunctionDeploy({ functionAppName: name, bundlePath, location: options.location });
            yield deploy.deploy();
            logger.info("Publishing complete");
        }
        catch (error) {
            logger.error(error);
            throw new Error("Error happenned while deploy the function app");
        }
    });
}
function deploy(name, projectPath, options) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info("Deploying project");
        yield pack(projectPath, options);
        yield publish(name, options);
        logger.info("Project deployed");
    });
}
runCli();
//# sourceMappingURL=main.js.map