"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const webpack = require("webpack");
const utils_1 = require("./utils");
const CONSTANTS_1 = require("./CONSTANTS");
const logger = utils_1.createLogger("azure.functions.webpack.WebpackRunner");
class WebpackRunner {
    static run(options) {
        options.indexFileName = options.indexFileName || CONSTANTS_1.DEFAULT_INDEX;
        options.outputPath = options.outputPath || CONSTANTS_1.DEFAULT_OUTPUT;
        options.uglify = options.uglify || false;
        options.ignoredModules = options.ignoredModules || [];
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            logger.debug("Setting up paths");
            const oldPath = path.join(options.projectRootPath, options.outputPath, options.indexFileName);
            const newPath = path.join(options.projectRootPath, options.outputPath, "original." + options.indexFileName);
            const outputPath = path.join(options.projectRootPath, options.outputPath, CONSTANTS_1.TEMP_OUTPUT);
            const ignoredModules = {};
            for (const mod of options.ignoredModules) {
                ignoredModules[mod.toLowerCase()] = mod;
            }
            logger.debug("Creating Webpack Configuration");
            const config = {
                entry: oldPath,
                externals: ignoredModules,
                node: {
                    __dirname: false,
                    __filename: false,
                },
                output: {
                    filename: CONSTANTS_1.TEMP_OUTPUT,
                    library: "index",
                    libraryTarget: "commonjs2",
                    path: path.join(options.projectRootPath, options.outputPath),
                },
                plugins: [],
                target: "node",
            };
            if (options.uglify) {
                logger.debug("Adding uglify plugin");
                try {
                    config.plugins.push(new webpack.optimize.UglifyJsPlugin());
                }
                catch (e) {
                    logger.error(e);
                }
            }
            logger.debug("Creating Webpack instance");
            const compiler = webpack(config);
            logger.debug("Started webpack");
            compiler.run((err, stats) => __awaiter(this, void 0, void 0, function* () {
                logger.debug("Webpack finished");
                if (err || stats.hasErrors()) {
                    return reject(err || stats.toString({ errors: true }));
                }
                logger.debug("\n" + stats.toString());
                logger.debug(`Saving the original the entry file: ${oldPath} -> ${newPath}"`);
                if (yield utils_1.FileHelper.exists(newPath)) {
                    yield utils_1.FileHelper.rimraf(newPath);
                }
                yield utils_1.FileHelper.rename(oldPath, newPath);
                logger.debug("Renaming the output file");
                yield utils_1.FileHelper.rename(outputPath, oldPath);
                resolve();
            }));
        }));
    }
}
exports.WebpackRunner = WebpackRunner;
//# sourceMappingURL=webpackRunner.js.map